 ;////////////////////////////////////////////////////////////////////////////
 ;
 ;  Copyright (C) Microsoft Corporation.  All Rights Reserved.
 ;
 ;  File:       d3dx9math.h
 ;  Content:    D3DX math types and functions
 ;
 ;////////////////////////////////////////////////////////////////////////////
IFNDEF __D3DX9_H__
INCLUDE d3dx9.INC
ENDIF
IFNDEF __D3DX9MATH_H__
__D3DX9MATH_H__ equ
;Include \masm32\sdk\math.SDK
IF COMPARE (_MSC_VER,GE,1200)
;ECHO   warning(push)
ENDIF 
;ECHO   warning(disable:4201)  ; anonymous unions warning
 ;===========================================================================
 ;
 ; General purpose utilities
 ;
 ;===========================================================================
D3DX_PI	equ	<3141592654f>
D3DX_1BYPI	equ	<0318309886f>
 ;===========================================================================
 ;
 ; 16 bit floating point numbers
 ;
 ;===========================================================================
D3DX_16F_DIG	equ	<3>
D3DX_16F_EPSILON	equ	<48875809e -  4f>
D3DX_16F_MANT_DIG	equ	<11>
D3DX_16F_MAX	equ	<6550400e +  004>
D3DX_16F_MAX_10_EXP	equ	<4>
D3DX_16F_MAX_EXP	equ	<15>
D3DX_16F_MIN	equ	<61035156e -  5f>
D3DX_16F_MIN_10_EXP	equ	< -4>
D3DX_16F_MIN_EXP	equ	< -12>
D3DX_16F_RADIX	equ	<2>
D3DX_16F_ROUNDS	equ	<1>
;------------------- Traduction Manuelle -----------------------------------
;RECAD3DXFLOAT16	RECORD public: 

;RECBD3DXFLOAT16	RECORD protected: 

 ;===========================================================================
 ;
 ; Vectors
 ;
 ;===========================================================================
 ;--------------------------
 ; 2D Vector
 ;--------------------------
;RECAD3DXVECTOR2	RECORD public: 

;RECBD3DXVECTOR2	RECORD public: 

 ;--------------------------
 ; 2D Vector (16 bit)
 ;--------------------------
;RECAD3DXVECTOR2_16F	RECORD public: 

;RECBD3DXVECTOR2_16F	RECORD public: 

 ;--------------------------
 ; 3D Vector
 ;--------------------------
IFDEF __cplusplus
;RECAD3DXVECTOR3	RECORD public: 

ELSE
ENDIF  ;!__cplusplus
 ;--------------------------
 ; 3D Vector (16 bit)
 ;--------------------------
;RECAD3DXVECTOR3_16F	RECORD public: 

;RECBD3DXVECTOR3_16F	RECORD public: 

 ;--------------------------
 ; 4D Vector
 ;--------------------------
;RECAD3DXVECTOR4	RECORD public: 

;RECBD3DXVECTOR4	RECORD public: 

 ;--------------------------
 ; 4D Vector (16 bit)
 ;--------------------------
;RECAD3DXVECTOR4_16F	RECORD public: 

;RECBD3DXVECTOR4_16F	RECORD public: 

 ;===========================================================================
 ;
 ; Matrices
 ;
 ;===========================================================================
IFDEF __cplusplus
;RECAD3DXMATRIX	RECORD public: 

ELSE
ENDIF  ;!__cplusplus
 ;---------------------------------------------------------------------------
 ; Aligned Matrices
 ;
 ; This class helps keep matrices 16-byte aligned as preferred by P4 cpus.
 ; It aligns matrices on the stack and on the heap or in global scope.
 ; It does this using __declspec(align(16)) which works on VC7 and on VC 6
 ; with the processor pack. Unfortunately there is no way to detect the
 ; latter so this is turned on only on VC7. On other compilers this is the
 ; the same as D3DXMATRIX.
 ;
 ; Using this class on a compiler that does not actually do the alignment
 ; can be dangerous since it will not expose bugs that ignore alignment.
 ; E.g if an object of this class in inside a struct or class, and some code
 ; memcopys data in it assuming tight packing. This could break on a compiler
 ; that eventually start aligning the matrix.
 ;---------------------------------------------------------------------------
IFDEF __cplusplus
ELSE
ENDIF  ;!__cplusplus
IF COMPARE (_MSC_VER,GE,1300) ; VC7
ELSE
D3DX_ALIGN16	equ	<>
ENDIF 
 ;===========================================================================
 ;
 ;    Quaternions
 ;
 ;===========================================================================
;RECAD3DXQUATERNION	RECORD public: 

 ;===========================================================================
 ;
 ; Planes
 ;
 ;===========================================================================
;RECAD3DXPLANE	RECORD public: 

 ;===========================================================================
 ;
 ; Colors
 ;
 ;===========================================================================
;RECAD3DXCOLOR	RECORD public: 

 ;===========================================================================
 ;
 ; D3DX math functions:
 ;
 ; NOTE:
 ;  * All these functions can take the same object as in and out parameters.
 ;
 ;  * Out parameters are typically also returned as return values, so that
 ;    the output of one function may be used as a parameter to another.
 ;
 ;===========================================================================
 ;--------------------------
 ; Float16
 ;--------------------------
 ; non-inline
IFDEF __cplusplus
ENDIF 
 ; Converts an array 32-bit floats to 16-bit floats
;API
D3DXFloat32To16Array PROTO :DWORD ,:DWORD ,:DWORD 
 ; Converts an array 16-bit floats to 32-bit floats
;API
D3DXFloat16To32Array PROTO :DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; 2D Vector
 ;--------------------------
 ; inline
D3DXVec2Length PROTO :DWORD 
D3DXVec2LengthSq PROTO :DWORD 
D3DXVec2Dot PROTO :DWORD ,:DWORD 
 ; Z component of ((x1,y1,0) cross (x2,y2,0))
D3DXVec2CCW PROTO :DWORD ,:DWORD 
D3DXVec2Add PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec2Subtract PROTO :DWORD ,:DWORD ,:DWORD 
 ; Minimize each component.  x = min(x1, x2), y = min(y1, y2)
D3DXVec2Minimize PROTO :DWORD ,:DWORD ,:DWORD 
 ; Maximize each component.  x = max(x1, x2), y = max(y1, y2)
D3DXVec2Maximize PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec2Scale PROTO :DWORD ,:DWORD ,:DWORD 
 ; Linear interpolation. V1 + s(V2-V1)
D3DXVec2Lerp PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
;API
D3DXVec2Normalize PROTO :DWORD ,:DWORD 
 ; Hermite interpolation between position V1, tangent T1 (when s == 0)
 ; and position V2, tangent T2 (when s == 1).
;API
D3DXVec2Hermite PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; CatmullRom interpolation between V1 (when s == 0) and V2 (when s == 1)
;API
D3DXVec2CatmullRom PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Barycentric coordinates.  V1 + f(V2-V1) + g(V3-V1)
;API
D3DXVec2BaryCentric PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, 0, 1) by matrix.
;API
D3DXVec2Transform PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, 0, 1) by matrix, project result back into w=1.
;API
D3DXVec2TransformCoord PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, 0, 0) by matrix.
;API
D3DXVec2TransformNormal PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform Array (x, y, 0, 1) by matrix.
;API
D3DXVec2TransformArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform Array (x, y, 0, 1) by matrix, project result back into w=1.
;API
D3DXVec2TransformCoordArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform Array (x, y, 0, 0) by matrix.
;API
D3DXVec2TransformNormalArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; 3D Vector
 ;--------------------------
 ; inline
D3DXVec3Length PROTO :DWORD 
D3DXVec3LengthSq PROTO :DWORD 
D3DXVec3Dot PROTO :DWORD ,:DWORD 
D3DXVec3Cross PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec3Add PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec3Subtract PROTO :DWORD ,:DWORD ,:DWORD 
 ; Minimize each component.  x = min(x1, x2), y = min(y1, y2), ...
D3DXVec3Minimize PROTO :DWORD ,:DWORD ,:DWORD 
 ; Maximize each component.  x = max(x1, x2), y = max(y1, y2), ...
D3DXVec3Maximize PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec3Scale PROTO :DWORD ,:DWORD ,:DWORD 
 ; Linear interpolation. V1 + s(V2-V1)
D3DXVec3Lerp PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
;API
D3DXVec3Normalize PROTO :DWORD ,:DWORD 
 ; Hermite interpolation between position V1, tangent T1 (when s == 0)
 ; and position V2, tangent T2 (when s == 1).
;API
D3DXVec3Hermite PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; CatmullRom interpolation between V1 (when s == 0) and V2 (when s == 1)
;API
D3DXVec3CatmullRom PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Barycentric coordinates.  V1 + f(V2-V1) + g(V3-V1)
;API
D3DXVec3BaryCentric PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, z, 1) by matrix.
;API
D3DXVec3Transform PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, z, 1) by matrix, project result back into w=1.
;API
D3DXVec3TransformCoord PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, z, 0) by matrix.  If you transforming a normal by a
 ; non-affine matrix, the matrix you pass to this function should be the
 ; transpose of the inverse of the matrix you would use to transform a coord.
;API
D3DXVec3TransformNormal PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform Array (x, y, z, 1) by matrix.
;API
D3DXVec3TransformArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform Array (x, y, z, 1) by matrix, project result back into w=1.
;API
D3DXVec3TransformCoordArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform (x, y, z, 0) by matrix.  If you transforming a normal by a
 ; non-affine matrix, the matrix you pass to this function should be the
 ; transpose of the inverse of the matrix you would use to transform a coord.
;API
D3DXVec3TransformNormalArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Project vector from object space into screen space
;API
D3DXVec3Project PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Project vector from screen space into object space
;API
D3DXVec3Unproject PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Project vector Array from object space into screen space
;API
D3DXVec3ProjectArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Project vector Array from screen space into object space
;API
D3DXVec3UnprojectArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; 4D Vector
 ;--------------------------
 ; inline
D3DXVec4Length PROTO :DWORD 
D3DXVec4LengthSq PROTO :DWORD 
D3DXVec4Dot PROTO :DWORD ,:DWORD 
D3DXVec4Add PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec4Subtract PROTO :DWORD ,:DWORD ,:DWORD 
 ; Minimize each component.  x = min(x1, x2), y = min(y1, y2), ...
D3DXVec4Minimize PROTO :DWORD ,:DWORD ,:DWORD 
 ; Maximize each component.  x = max(x1, x2), y = max(y1, y2), ...
D3DXVec4Maximize PROTO :DWORD ,:DWORD ,:DWORD 
D3DXVec4Scale PROTO :DWORD ,:DWORD ,:DWORD 
 ; Linear interpolation. V1 + s(V2-V1)
D3DXVec4Lerp PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
 ; Cross-product in 4 dimensions.
;API
D3DXVec4Cross PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
;API
D3DXVec4Normalize PROTO :DWORD ,:DWORD 
 ; Hermite interpolation between position V1, tangent T1 (when s == 0)
 ; and position V2, tangent T2 (when s == 1).
;API
D3DXVec4Hermite PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; CatmullRom interpolation between V1 (when s == 0) and V2 (when s == 1)
;API
D3DXVec4CatmullRom PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Barycentric coordinates.  V1 + f(V2-V1) + g(V3-V1)
;API
D3DXVec4BaryCentric PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform vector by matrix.
;API
D3DXVec4Transform PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform vector array by matrix.
;API
D3DXVec4TransformArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; 4D Matrix
 ;--------------------------
 ; inline
D3DXMatrixIdentity PROTO :DWORD 
D3DXMatrixIsIdentity PROTO :DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
;API
D3DXMatrixDeterminant PROTO :DWORD 
;API
D3DXMatrixTranspose PROTO :DWORD ,:DWORD 
 ; Matrix multiplication.  The result represents the transformation M2
 ; followed by the transformation M1.  (Out = M1 * M2)
;API
D3DXMatrixMultiply PROTO :DWORD ,:DWORD ,:DWORD 
 ; Matrix multiplication, followed by a transpose. (Out = T(M1 * M2))
;API
D3DXMatrixMultiplyTranspose PROTO :DWORD ,:DWORD ,:DWORD 
 ; Calculate inverse of matrix.  Inversion my fail, in which case NULL will
 ; be returned.  The determinant of pM is also returned it pfDeterminant
 ; is non-NULL.
;API
D3DXMatrixInverse PROTO :DWORD ,:DWORD ,:DWORD 
 ; Build a matrix which scales by (sx, sy, sz)
;API
D3DXMatrixScaling PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a matrix which translates by (x, y, z)
;API
D3DXMatrixTranslation PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a matrix which rotates around the X axis
;API
D3DXMatrixRotationX PROTO :DWORD ,:DWORD 
 ; Build a matrix which rotates around the Y axis
;API
D3DXMatrixRotationY PROTO :DWORD ,:DWORD 
 ; Build a matrix which rotates around the Z axis
;API
D3DXMatrixRotationZ PROTO :DWORD ,:DWORD 
 ; Build a matrix which rotates around an arbitrary axis
;API
D3DXMatrixRotationAxis PROTO :DWORD ,:DWORD ,:DWORD 
 ; Build a matrix from a quaternion
;API
D3DXMatrixRotationQuaternion PROTO :DWORD ,:DWORD 
 ; Yaw around the Y axis, a pitch around the X axis,
 ; and a roll around the Z axis.
;API
D3DXMatrixRotationYawPitchRoll PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build transformation matrix.  NULL arguments are treated as identity.
 ; Mout = Msc-1 * Msr-1 * Ms * Msr * Msc * Mrc-1 * Mr * Mrc * Mt
;API
D3DXMatrixTransformation PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build affine transformation matrix.  NULL arguments are treated as identity.
 ; Mout = Ms * Mrc-1 * Mr * Mrc * Mt
;API
D3DXMatrixAffineTransformation PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a lookat matrix. (right-handed)
;API
D3DXMatrixLookAtRH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a lookat matrix. (left-handed)
;API
D3DXMatrixLookAtLH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a perspective projection matrix. (right-handed)
;API
D3DXMatrixPerspectiveRH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a perspective projection matrix. (left-handed)
;API
D3DXMatrixPerspectiveLH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a perspective projection matrix. (right-handed)
;API
D3DXMatrixPerspectiveFovRH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a perspective projection matrix. (left-handed)
;API
D3DXMatrixPerspectiveFovLH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a perspective projection matrix. (right-handed)
;API
D3DXMatrixPerspectiveOffCenterRH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a perspective projection matrix. (left-handed)
;API
D3DXMatrixPerspectiveOffCenterLH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build an ortho projection matrix. (right-handed)
;API
D3DXMatrixOrthoRH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build an ortho projection matrix. (left-handed)
;API
D3DXMatrixOrthoLH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build an ortho projection matrix. (right-handed)
;API
D3DXMatrixOrthoOffCenterRH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build an ortho projection matrix. (left-handed)
;API
D3DXMatrixOrthoOffCenterLH PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Build a matrix which flattens geometry into a plane, as if casting
 ; a shadow from a light.
;API
D3DXMatrixShadow PROTO :DWORD ,:DWORD ,:DWORD 
 ; Build a matrix which reflects the coordinate system about a plane
;API
D3DXMatrixReflect PROTO :DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; Quaternion
 ;--------------------------
 ; inline
D3DXQuaternionLength PROTO :DWORD 
 ; Length squared, or "norm"
D3DXQuaternionLengthSq PROTO :DWORD 
D3DXQuaternionDot PROTO :DWORD ,:DWORD 
 ; (0, 0, 0, 1)
D3DXQuaternionIdentity PROTO :DWORD 
D3DXQuaternionIsIdentity PROTO :DWORD 
 ; (-x, -y, -z, w)
D3DXQuaternionConjugate PROTO :DWORD ,:DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
 ; Compute a quaternin's axis and angle of rotation. Expects unit quaternions.
;API
D3DXQuaternionToAxisAngle PROTO :DWORD ,:DWORD ,:DWORD 
 ; Build a quaternion from a rotation matrix.
;API
D3DXQuaternionRotationMatrix PROTO :DWORD ,:DWORD 
 ; Rotation about arbitrary axis.
;API
D3DXQuaternionRotationAxis PROTO :DWORD ,:DWORD ,:DWORD 
 ; Yaw around the Y axis, a pitch around the X axis,
 ; and a roll around the Z axis.
;API
D3DXQuaternionRotationYawPitchRoll PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Quaternion multiplication.  The result represents the rotation Q2
 ; followed by the rotation Q1.  (Out = Q2 * Q1)
;API
D3DXQuaternionMultiply PROTO :DWORD ,:DWORD ,:DWORD 
;API
D3DXQuaternionNormalize PROTO :DWORD ,:DWORD 
 ; Conjugate and re-norm
;API
D3DXQuaternionInverse PROTO :DWORD ,:DWORD 
 ; Expects unit quaternions.
 ; if q = (cos(theta), sin(theta) * v); ln(q) = (0, theta * v)
;API
D3DXQuaternionLn PROTO :DWORD ,:DWORD 
 ; Expects pure quaternions. (w == 0)  w is ignored in calculation.
 ; if q = (0, theta * v); exp(q) = (cos(theta), sin(theta) * v)
;API
D3DXQuaternionExp PROTO :DWORD ,:DWORD 
 ; Spherical linear interpolation between Q1 (t == 0) and Q2 (t == 1).
 ; Expects unit quaternions.
;API
D3DXQuaternionSlerp PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Spherical quadrangle interpolation.
 ; Slerp(Slerp(Q1, C, t), Slerp(A, B, t), 2t(1-t))
;API
D3DXQuaternionSquad PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Setup control points for spherical quadrangle interpolation
 ; from Q1 to Q2.  The control points are chosen in such a way
 ; to ensure the continuity of tangents with adjacent segments.
;API
D3DXQuaternionSquadSetup PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Barycentric interpolation.
 ; Slerp(Slerp(Q1, Q2, f+g), Slerp(Q1, Q3, f+g), g/(f+g))
;API
D3DXQuaternionBaryCentric PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; Plane
 ;--------------------------
 ; inline
 ; ax + by + cz + dw
D3DXPlaneDot PROTO :DWORD ,:DWORD 
 ; ax + by + cz + d
D3DXPlaneDotCoord PROTO :DWORD ,:DWORD 
 ; ax + by + cz
D3DXPlaneDotNormal PROTO :DWORD ,:DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
 ; Normalize plane (so that |a,b,c| == 1)
;API
D3DXPlaneNormalize PROTO :DWORD ,:DWORD 
 ; Find the intersection between a plane and a line.  If the line is
 ; parallel to the plane, NULL is returned.
;API
D3DXPlaneIntersectLine PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Construct a plane from a point and a normal
;API
D3DXPlaneFromPointNormal PROTO :DWORD ,:DWORD ,:DWORD 
 ; Construct a plane from 3 points
;API
D3DXPlaneFromPoints PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; Transform a plane by a matrix.  The vector (a,b,c) must be normal.
 ; M should be the inverse transpose of the transformation desired.
;API
D3DXPlaneTransform PROTO :DWORD ,:DWORD ,:DWORD 
 ; Transform an array of planes by a matrix.  The vectors (a,b,c) must be normal.
 ; M should be the inverse transpose of the transformation desired.
;API
D3DXPlaneTransformArray PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; Color
 ;--------------------------
 ; inline
 ; (1-r, 1-g, 1-b, a)
D3DXColorNegative PROTO :DWORD ,:DWORD 
D3DXColorAdd PROTO :DWORD ,:DWORD ,:DWORD 
D3DXColorSubtract PROTO :DWORD ,:DWORD ,:DWORD 
D3DXColorScale PROTO :DWORD ,:DWORD ,:DWORD 
 ; (r1*r2, g1*g2, b1*b2, a1*a2)
D3DXColorModulate PROTO :DWORD ,:DWORD ,:DWORD 
 ; Linear interpolation of r,g,b, and a. C1 + s(C2-C1)
D3DXColorLerp PROTO :DWORD ,:DWORD ,:DWORD ,:DWORD 
 ; non-inline
IFDEF __cplusplus
ENDIF 
 ; Interpolate r,g,b between desaturated color and color.
 ; DesaturatedColor + s(Color - DesaturatedColor)
;API
D3DXColorAdjustSaturation PROTO :DWORD ,:DWORD ,:DWORD 
 ; Interpolate r,g,b between 50% grey and color.  Grey + s(Color - Grey)
;API
D3DXColorAdjustContrast PROTO :DWORD ,:DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;--------------------------
 ; Misc
 ;--------------------------
IFDEF __cplusplus
ENDIF 
 ; Calculate Fresnel term given the cosine of theta (likely obtained by
 ; taking the dot of two normals), and the refraction index of the material.
;API
D3DXFresnelTerm PROTO :DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
 ;===========================================================================
 ;
 ;    Matrix Stack
 ;
 ;===========================================================================
 ; {E3357330-CC5E-11d2-A434-00A0C90629A8}
sIID_ID3DXMatrixStack	TEXTEQU	<{0e3357330h,0cc5eh,011d2h,(0a4h,034h,00h,0a0h,0c9h,06h,029h,0a8h}}>
INTERFACE	equ	<ID3DXMatrixStack>
;ID3DXMatrixStack drive de : IUnknown
	STID3DXMatrixStack	STRUCT
		QueryInterface               	comethod3 ?
		AddRef                       	comethod1 ?
		Release                      	comethod1 ?
		Pop1                          	comethod1 ?
		Push1                         	comethod1 ?
		LoadIdentity                 	comethod1 ?
		LoadMatrix                   	comethod2 ?
		MultMatrix                   	comethod2 ?
		MultMatrixLocal              	comethod2 ?
		RotateAxis                   	comethod3 ?
		RotateAxisLocal              	comethod3 ?
		RotateYawPitchRoll           	comethod4 ?
		RotateYawPitchRollLocal      	comethod4 ?
		Scale                        	comethod4 ?
		ScaleLocal                   	comethod4 ?
		Translate                    	comethod4 ?
		TranslateLocal               	comethod4 ?
		GetTop                       	comethod1 ?
	STID3DXMatrixStack	ENDS
 
ID3DXMatrixStack MACRO  Function:REQ, args:VARARG
	; definition de la macro locale InvokeInterface
    LOCAL InvokeInterface, arg
    FOR arg, <args>     ;verifier que edx n'est pas dans la liste d'arguments args
        IFIDNI <&arg>, <edx>	;
            .ERR <edx is not allowed as a coinvoke parameter>
        ENDIF
    ENDM
    IFIDNI <&pInterface>, <edx>
        .ERR <edx is not allowed as a coinvoke parameter>
    ENDIF
	;InvokeInterface = concatene ...CATSTR(concatene) MACRO instruction MASM32	
	;---------- on doit mettre ppv en premier argument -----------------------------------
    InvokeInterface CATSTR <invoke (STID3DXMatrixStack PTR[edx]).>,<&Function,ppvID3DXMatrixStack>
    IFNB <args>     ; add the list of parameter arguments if any
        InvokeInterface CATSTR InvokeInterface, <, >, <&args> 
    ENDIF 
	;   forme les lignes de codes
    mov edx, ppvID3DXMatrixStack
    mov edx, [edx]
    InvokeInterface
ENDM		
 
 
IFDEF __cplusplus
ENDIF 
;API
D3DXCreateMatrixStack PROTO :DWORD ,:DWORD 
IFDEF __cplusplus
ENDIF 
IF COMPARE (_MSC_VER,GE,1200)
;ECHO   warning(pop)
ELSE
;ECHO   warning(default:4201)
ENDIF 
ENDIF  ; __D3DX9MATH_H__
